module uart_ctrl(
	clk,
	rst_n,
	uart_rxd,
	uart_done,
	rxd_flag,
	rxd_cnt,
	rxd_date,
	uart_date);

	//---Ports declearation: generated by Robei---
	input clk;
	input rst_n;
	input uart_rxd;
	output uart_done;
	output rxd_flag;
	output [3:0] rxd_cnt;
	output [7:0] rxd_date;
	output [7:0] uart_date;

	wire clk;
	wire rst_n;
	wire uart_rxd;
	reg uart_done;
	reg rxd_flag;
	reg [3:0] rxd_cnt;
	reg [7:0] rxd_date;
	reg [7:0] uart_date;

	//----Code starts here: integrated by Robei-----
	parameter  clk_freq = 50_000_000;
	parameter  uart_bps = 9600;
	localparam bps_cnt  = clk_freq/uart_bps;
	
	
	reg [15:0] clk_cnt;
	reg        uart_rxd0;
	reg        uart_rxd1;
	
	
	wire       start_flag;
	
	assign start_flag = uart_rxd1 & (~uart_rxd0);
	
	
	always @(posedge clk or negedge rst_n)begin
	    if(!rst_n)begin
	        uart_rxd0 <= 1'b0;
	        uart_rxd1 <= 1'b0;
	    end
	    else begin
	        uart_rxd0 <= uart_rxd;
	        uart_rxd1 <= uart_rxd0;
	    end
	end 
	
	always @(posedge clk or negedge rst_n)begin
	    if(!rst_n)
	        rxd_flag <= 1'b0;
	    else begin
	        if(start_flag)
	            rxd_flag <= 1'b1;
	        else if ((rxd_cnt == 4'd9) && (clk_cnt == bps_cnt/2))
	            rxd_flag <= 1'b0;
	        else 
	            rxd_flag <= rxd_flag;
	    end    
	end
	
	
	always @(posedge clk or negedge rst_n)begin
	    if(!rst_n)
	        clk_cnt <= 16'd0;
	    else if(rxd_flag)begin
	        if(clk_cnt < bps_cnt - 1)
	            clk_cnt <= clk_cnt + 1'b1;
	        else
	            clk_cnt <= 16'd0;
	    end
	    else 
	        clk_cnt <= 16'd0;
	end
	
	always @(posedge clk or negedge rst_n)begin
	    if(!rst_n)
	        rxd_cnt <= 4'd0;
	    else if(rxd_flag)begin
	        if(clk_cnt == bps_cnt - 1)
	            rxd_cnt <= rxd_cnt + 1'b1;
	        else
	            rxd_cnt <= rxd_cnt;
	    end
	    else 
	        rxd_cnt <= 4'd0;
	end
	
	always @(posedge clk or negedge rst_n)begin
	    if(!rst_n)
	        rxd_date <= 8'd0;
	    else if(rxd_flag) begin
	        if(clk_cnt == uart_bps/2)begin
	            case(rxd_cnt)
	             4'd1 : rxd_date[0] <= uart_rxd1;
	             4'd2 : rxd_date[1] <= uart_rxd1;
	             4'd3 : rxd_date[2] <= uart_rxd1;
	             4'd4 : rxd_date[3] <= uart_rxd1;
	             4'd5 : rxd_date[4] <= uart_rxd1;
	             4'd6 : rxd_date[5] <= uart_rxd1;
	             4'd7 : rxd_date[6] <= uart_rxd1;
	             4'd8 : rxd_date[7] <= uart_rxd1; 
	             default :;
	             endcase
	        end
	        else
	            rxd_date <= rxd_date;
	    end
	    else 
	             rxd_date <= 8'd0;
	end
	
	always @(posedge clk or negedge rst_n)begin
	    if(!rst_n)begin
	        uart_date <= 8'd0;
	        uart_done <= 1'b0;
	    end
	    else if(rxd_cnt == 4'd9)begin
	        uart_date <= rxd_date;
	        uart_done <= 1'b1;
	    end
	    else begin
	        uart_date <= 8'd0;
	        uart_done <= 1'b0;
	    end
	end
	
	
	
endmodule    //uart_ctrl

